@@ -13,13 +13,16 @@ use output::*;
 use comfy_table::{Cell, Table};
 
 use repo::{
-    clone_repo, detect_remote_type, get_repo_status, init_repo, open_repo, Remote,
-    RemoteTrackingStatus, Repo, RepoErrorKind,
+    clone_repo, detect_remote_type, get_repo_status, init_repo, open_repo, repo_make_bare,
+    repo_set_config_push, Remote, RemoteTrackingStatus, Repo, RepoErrorKind,
 };
 
 const GIT_MAIN_WORKTREE_DIRECTORY: &str = ".git-main-working-tree";
 const BRANCH_NAMESPACE_SEPARATOR: &str = "/";
 
+const GIT_CONFIG_BARE_KEY: &str = "core.bare";
+const GIT_CONFIG_PUSH_DEFAULT: &str = "push.default";
+
 #[cfg(test)]
 mod tests {
     use super::*;
@@ -631,7 +634,7 @@ fn show_single_repo_status(path: &Path, is_worktree: bool) {
 
     if let Err(error) = repo_handle {
         if error.kind == RepoErrorKind::NotFound {
-            print_error(&"Directory is not a git directory".to_string());
+            print_error("Directory is not a git directory");
         } else {
             print_error(&format!("Opening repository failed: {}", error));
         }
@@ -868,28 +871,33 @@ pub fn run() {
                 }
             };
 
-            let repo = match open_repo(&dir, true) {
-                Ok(r) => r,
-                Err(e) => {
-                    match e.kind {
-                        RepoErrorKind::NotFound => {
-                            print_error("Current directory does not contain a worktree setup")
+            fn get_repo(dir: &Path) -> git2::Repository {
+                match open_repo(dir, true) {
+                    Ok(r) => r,
+                    Err(e) => {
+                        match e.kind {
+                            RepoErrorKind::NotFound => {
+                                print_error("Current directory does not contain a worktree setup")
+                            }
+                            _ => print_error(&format!("Error opening repo: {}", e)),
                         }
-                        _ => print_error(&format!("Error opening repo: {}", e)),
+                        process::exit(1);
                     }
-                    process::exit(1);
                 }
-            };
+            }
 
-            let worktrees = repo
-                .worktrees()
-                .unwrap()
-                .iter()
-                .map(|e| e.unwrap().to_string())
-                .collect::<Vec<String>>();
+            fn get_worktrees(repo: &git2::Repository) -> Vec<String> {
+                repo.worktrees()
+                    .unwrap()
+                    .iter()
+                    .map(|e| e.unwrap().to_string())
+                    .collect::<Vec<String>>()
+            }
 
             match args.action {
                 cmd::WorktreeAction::Add(action_args) => {
+                    let repo = get_repo(&dir);
+                    let worktrees = get_worktrees(&repo);
                     if worktrees.contains(&action_args.name) {
                         print_error("Worktree already exists");
                         process::exit(1);
@@ -1014,6 +1022,7 @@ pub fn run() {
 
                 cmd::WorktreeAction::Delete(action_args) => {
                     let worktree_dir = dir.join(&action_args.name);
+                    let repo = get_repo(&dir);
 
                     match remove_worktree(
                         &action_args.name,
@@ -1040,6 +1049,8 @@ pub fn run() {
                     }
                 }
                 cmd::WorktreeAction::Status(_args) => {
+                    let repo = get_repo(&dir);
+                    let worktrees = get_worktrees(&repo);
                     let mut table = Table::new();
                     add_worktree_table_header(&mut table);
                     for worktree in &worktrees {
@@ -1081,8 +1092,100 @@ pub fn run() {
                     }
                     println!("{}", table);
                 }
+                cmd::WorktreeAction::Convert(_args) => {
+                    // Converting works like this:
+                    // * Check whether there are uncommitted/unpushed changes
+                    // * Move the contents of .git dir to the worktree directory
+                    // * Remove all files
+                    // * Set `core.bare` to `true`
+
+                    let repo = open_repo(&dir, false).unwrap_or_else(|error| {
+                        if error.kind == RepoErrorKind::NotFound {
+                            print_error("Directory does not contain a git repository");
+                        } else {
+                            print_error(&format!("Opening repository failed: {}", error));
+                        }
+                        process::exit(1);
+                    });
+
+                    let status = get_repo_status(&repo, false);
+                    if status.changes.unwrap().is_some() {
+                        print_error("Changes found in repository, refusing to convert");
+                    }
+
+                    if let Err(error) = std::fs::rename(".git", GIT_MAIN_WORKTREE_DIRECTORY) {
+                        print_error(&format!("Error moving .git directory: {}", error));
+                    }
+
+                    for entry in match std::fs::read_dir(&dir) {
+                        Ok(iterator) => iterator,
+                        Err(error) => {
+                            print_error(&format!("Opening directory failed: {}", error));
+                            process::exit(1);
+                        }
+                    } {
+                        match entry {
+                            Ok(entry) => {
+                                let path = entry.path();
+                                // The path will ALWAYS have a file component
+                                if path.file_name().unwrap() == GIT_MAIN_WORKTREE_DIRECTORY {
+                                    continue;
+                                }
+                                if path.is_file() || path.is_symlink() {
+                                    if let Err(error) = std::fs::remove_file(&path) {
+                                        print_error(&format!("Failed removing {}", error));
+                                        process::exit(1);
+                                    }
+                                } else if let Err(error) = std::fs::remove_dir_all(&path) {
+                                    print_error(&format!("Failed removing {}", error));
+                                    process::exit(1);
+                                }
+                            }
+                            Err(error) => {
+                                print_error(&format!("Error getting directory entry: {}", error));
+                                process::exit(1);
+                            }
+                        }
+                    }
+
+                    let worktree_repo = open_repo(&dir, true).unwrap_or_else(|error| {
+                        print_error(&format!(
+                            "Opening newly converted repository failed: {}",
+                            error
+                        ));
+                        process::exit(1);
+                    });
+
+                    repo_make_bare(&worktree_repo, true).unwrap_or_else(|error| {
+                        print_error(&format!("Error: {}", error));
+                        process::exit(1);
+                    });
+
+                    repo_set_config_push(&worktree_repo, "upstream").unwrap_or_else(|error| {
+                        print_error(&format!("Error: {}", error));
+                        process::exit(1);
+                    });
+
+                    print_success("Conversion done");
+                }
                 cmd::WorktreeAction::Clean(_args) => {
-                    for worktree in &worktrees {
+                    let repo = get_repo(&dir);
+                    let worktrees = get_worktrees(&repo);
+
+                    let default_branch = match get_default_branch(&repo) {
+                        Ok(branch) => branch,
+                        Err(error) => {
+                            print_error(&format!("Failed getting default branch: {}", error));
+                            process::exit(1);
+                        }
+                    };
+
+                    let default_branch_name = default_branch.name().unwrap().unwrap();
+
+                    for worktree in worktrees
+                        .iter()
+                        .filter(|worktree| *worktree != default_branch_name)
+                    {
                         let repo_dir = &dir.join(&worktree);
                         if repo_dir.exists() {
                             match remove_worktree(worktree, repo_dir, false, &repo) {
@@ -1108,6 +1211,7 @@ pub fn run() {
                             ));
                         }
                     }
+
                     for entry in std::fs::read_dir(&dir).unwrap() {
                         let dirname = path_as_string(
                             &entry
@@ -1120,6 +1224,9 @@ pub fn run() {
                         if dirname == GIT_MAIN_WORKTREE_DIRECTORY {
                             continue;
                         }
+                        if dirname == default_branch_name {
+                            continue;
+                        }
                         if !&worktrees.contains(&dirname) {
                             print_warning(&format!(
                                 "Found {}, which is not a valid worktree directory!",
